
/*
  SCRIPTOL LIBRARY
  (c) 2002-2003 by Denis G. Sureau
  http://www.scriptol.com

LICENCE

Along with the free Scriptol to native compiler is provided the libsol.a
or libsol.lib library, it contains the TEXT, DYN and ARRAY classes and
lot of functions compatibles with the Php library.
This library is the original work of Denis G. Sureau and has been
registered at INPI 25 june 2002. The Php compatible functions are original
work also. They have nothing to do with Php sources.

The library is provided to allow the user to link it with its Scriptol
compiled program. He is allowed to include the library in binary form
into its program and distribute its programs with the library included
into, with no restriction or condition or fee due.

He is not allowed to distribute the compiled library as it, but as a part
of an open-source program. The compiled library may be included among
other files of the open source distribution and headers of the file to
include, that must hold my name and the address of the Scriptol site.

The source of the library is provided to users to allow them either to
modify it according to their need, or to compile it in a different file
format, for a different compiler. The users are not permitted to distribute
the source of the library in no any case, even modified.

*/


#if !defined(_SOL_FUN)
#define _SOL_FUN

#define STR_PAD_LEFT 0
#define STR_PAD_RIGHT 1
#define STR_PAD_BOTH 2

#define PHP_VERSION "C++"
#define PHP_OS "unknow"



//====== internal, replace dirent.h for all including visual c

DIR*            opendir (const char*);
struct dirent*  readdir (DIR*);
int             closedir (DIR*);
void            rewinddir (DIR*);
long            telldir (DIR*);
void            seekdir (DIR*, long);

//===== general

/**
  Displaying an array
  */

#define print_r(a) a.display()

const char *plural(int);

void runtimeError(const char *);

/**
  Displaying a message before to exit
  */
void die(text);
void die(const char *);

/**
  Stop the program, return to operating system
  */
void exit();

/**
  Swap the content of two variables.
  */
#define swap(a,b) { dyn temp = a; a = b; b = temp; }

/**
  Get the minimum of two values
  */
int min(int, int);
natural min(natural, natural);
double min(double, double);
dyn min(dyn, dyn);
text min(text, text);

/**
  Get the maximum of two values
  */
int max(int, int);
natural max(natural, natural);
double max(double, double);
dyn max(dyn, dyn);
text max(text, text);

//===== types

/**
  Return true if the argument is an associative array.
  */
bool is_array(dyn a);

/**
  Return true if the argument is a typed array.
  */
bool is_list(dyn a);

//===== numbers

/**
  Convert the argument to natural and return it.
  The argument must be a scalar.
  */
natural naturalval(dyn);
natural naturalval(unsigned char);
natural naturalval(int);
natural naturalval(natural);
natural naturalval(float);
natural naturalval(double);
natural naturalval(text);
natural naturalval(const char *);
natural naturalval(void *);

/**
  Convert the argument to number and return it.
  The argument must be a scalar.
  */
double doubleval(dyn);
double doubleval(text);
double doubleval(const char *);
double doubleval(int);
double doubleval(natural);
double doubleval(double);
double doubleval(void *);

/**
  Convert the argument to integer and return it.
  The argument must be a scalar.
  */
int intval(dyn);
int intval(text);
int intval(const char *);
int intval(double);
int intval(int);
int intval(natural);
int intval(void *);

/**
  Convert the argument to a C string of char and return it.
  The argument must be a scalar.
  */
char *str(int);
char *str(natural);
char *str(double);
char *str(dyn);
char *str(void *);

/**
  Convert the argument to a text and return it.
  The argument must be a scalar.
  */
text strval(int);
text strval(natural);
text strval(double);
text strval(dyn);
text strval(void *);
text strval(bool);

/**
  Convert the int to bool
  */

bool boolval(int);

/**
  Return the text holding an integer as a pure integer.
  The argument must be a text.
  */
int ord(const char *);
int ord(text);
int ord(char);

/**
  Return the integer argument as a text.
  */
text chr(int);

double _sol_round(double);

//===== array

#define array_values(a) a.values()
#define array_keys(a) a.keys()
#define array_flip(a) a.flip()

int   count(array);

array _sol_range(int, int);

array array_merge(array, array);
array array_diff(array , array);
array array_intersect(array, array);
array array_unique(const array &);
int array_search(array, ...);
array array_reverse(array, bool);

text implode(array *, text sep);

//template<class R> void array_map(R(*)(dyn &), array &);
//template<class R> void array_map(R(*)(dyn *), array &);
void array_map(void (*)(dyn &), array &);
void array_map(void (*)(dyn *), array &);
void array_map(void (*fun)(dyn &, dyn &), array &, array &);
void array_map(void (*fun)(dyn &, dyn &, dyn &), array &, array &, array &);



bool in_array(int, array);
bool in_array(text, array);
bool in_array(const char *, array);
bool in_array(double, array);
bool in_array(dyn, array);

bool in_sequence(const char *, text &);
bool in_sequence(const char *, const char *);
//bool in_sequence(dyn, array);
//bool in_sequence(text, array);

//===== iterators

dyn reset(array);
dyn next(array);
dyn prev(array);
dyn end(array);
text key(array &);
dyn current(array &);


//===== stack

#define array_unshift(a, d) a.unshift(d)
#define array_shift(a) a.shift()
#define array_pop(a) a.pop()
#define array_push(a, d)  a.push(d)

//===== functions

text implode(text, array);


//===== file

array file(text);
array file(const char *);
void store(array, const char *);
void store(array, text);
void require(const char *);
void require_once(const char *);

//===== text


char *builtin_hex(int);


char *strtoupper(const char *);
char *strtolower(const char *);
char *ucfirst(const char *);
char *ucwords(const char *);
char *wordwrap(const char *, int, const char *);

int strlen(text);
int strlen(dyn);
int strcasecmp(text , text );
void str_replace(text , text );

char *substr(const char *, int, int);
char *substr(const char *, int);
int substr_count(const char *, const char *);
char *substr_replace(const char *, const char *, int, int);

#ifdef _UNIX
//char *strstr(const char *, const char *);
int strcasecmp(const char *, const char *);
#endif

char *stristr(const char *, const char *);

char *str_replace(const char *, const char *, const char *);
char *strtr(const char *, const char *, const char *);

int strpos(const char *, const char *, int);

array explode(const char *, const char *);

char *chop(const char *);
char *rtrim(const char *);
char *ltrim(const char *);
char *trim(const char *);

text str_pad(text , int, text = " ", int = STR_PAD_RIGHT);
text str_pad(text , int, const char * =  " ", int = STR_PAD_RIGHT);
text str_pad(const char *, int, const char * = " ", int = STR_PAD_RIGHT);

char *str_repeat(const char *, int);

//===== file

FILE *fopen(text, text);
FILE *fopen(text, const char *);
FILE *_sol_open(const char *, const char *);
void fseek(FILE *, int);
inline void fputs(const text &t) { fputs(t.data); }
text fgets(FILE *);
text fread(FILE *, int size);
int  fwrite(FILE *, const text &);
int  fwrite(FILE *, const char *);

//===== dir

bool file_exists(const text &);
bool file_exists(const char *);
text filetype(char *);
double filesize(const char *);
int  filemtime(const char *);
const char *date(const char *, int);
bool _sol_chdir(const char *);          // change directory for a drive/current
bool _sol_mkdir(const char *);          // create directory
bool _sol_rmdir(const char *);          // remove directory
bool _sol_ren(const char *, const char *);   // rename file

dyn open_dir(char *);
text read_dir(dyn);
void rewind_dir(dyn);       // removed for now (not useful)
void close_dir(dyn);
text getcwd();


//===== html

char *extractProperty(const char *, const char *);
array get_meta_tags(const char *);

//===== regular not yet implemented

bool ereg(text t, text);
bool ereg(text t, const char *);
bool ereg(const char *, text);
bool ereg(const char *, const char *);

//====== Time

int time();

#endif



