
#ifndef _SOL_JVM_
#define _SOL_JVM_
#include "jni.h"

class jvm
{
  JavaVM *jvmach;
  JavaVMInitArgs vm_args;
  JavaVMOption options[4];

  bool isStatic;
  jclass staticClass;

  public:

  JNIEnv *env;

  jvm(char *);
  ~jvm();

/**
  Load the ini file of the program invoking Java
  (not to be confused with the .cfg file used by the compiler for this program).
  */

  void loadIni(char *);
  char *setEntry(char *, char *);
  void processEntry(char *);
  static bool verbose;
  static char *hpath;
  static char *cpath;
  static char *lpath;

/**
   Displays an error at Jvm call
  */

  void JVMError(int);

/**
  Builds a JavaString (Object String) out of a C string of chars.
  */

  jstring oj(const char *);
  inline jstring oj(text t) { return oj(t.data); }

/**
  Converts an array of string into a java array of javastrings.
  Input:  jclass number, array of strings, number of elements.
  Output: jobjectArray
 */

  jobjectArray BuildArrayOfStrings(jclass, char *[], int);

/**
  Get the ID of a java class
  Input: name of the class
  Output: the number
  */

  jclass FindClass(char *);

/**
  Get the ID of the instance of a java class
  Input: ID of the class (returned by FindClass)
  Ouput: ID of the object
  */

  jobject jvm::FindObject(jclass);

/**
  Get the ID for an attribute of a java class
  Input: identifier, descriptor
  Ouput: jfieldID
  */

  jfieldID GetField(jclass, const char *, const char *);

/**
  Get the ID for a method of a java class
  Input: identifier, descriptor
  Ouput: jmethodID
  */

  jmethodID GetMethod(jclass, const char *, const char *);
  jmethodID GetStaticMethod(jclass, const char *, const char *);

/**
  Get the value of an integer attribute of a class.
  Input: ID of the class, of the instance, name of the method.
  Output: integer value.
  */

  int GetInt(jclass, jobject, char *);
  char *GetString(jclass, jobject, char *);
  double GetNumber(jclass, jobject, char *);
  bool GetBoolean(jclass, jobject, char *);

/**
  Call a method of a class and return the value.
  Input: ID of the class, of the instance, name of the method.
  Output: integer value.
  */

//  bool CallBoolean(jclass, jobject, char *);
  int CallInt(jobject, jmethodID, ...);
//  int CallIntMethod(jclass, jobject, char *, char *, ...);

//  double CallNumber(jclass, jobject, char *);

  const char *CallString(jobject, jmethodID, ...);

  void CallVoid(jobject, jmethodID, ...);

  void *CallObject(jobject, jmethodID, ...);


};


extern jvm *_jvm;

class Java
{
  public:
  jclass classID;
  jobject instanceID;

  Java(const char *name)
  {
    classID = _jvm->FindClass((char *) name);
    instanceID = _jvm->FindObject(classID);
  }

};


#endif
