
/*
  SCRIPTOL LIBRARY
  (c) 2002 by Denis G. Sureau
  http://www.scriptol.com

LICENCE

Along with the free Scriptol to native compiler is provided the libsol.a
or libsol.lib library, it contains the TEXT, DYN and ARRAY classes and
lot of functions compatibles with the Php library.
This library is the original work of Denis G. Sureau and has been
registered at INPI 25 june 2002. The Php compatible functions are original
work also. They have nothing to do with Php sources.

The library is provided to allow the user to link it with its Scriptol
compiled program. He is allowed to include the library in binary form
into its program and distribute its programs with the library included
into, with no restriction or condition or fee due.

He is not allowed to distribute the compiled library as it, but as a part
of an open-source program. The compiled library may be included among
other files of the open source distribution and headers of the file to
include, that must hold my name and the address of the Scriptol site.

The source of the library is provided to users to allow them either to
modify it according to their need, or to compile it in a different file
format, for a different compiler. The users are not permitted to distribute
the source of the library in no any case, even modified.
*/



#if !defined(_SOL_TEXT)
#define _SOL_TEXT

#ifdef UNIX
typedef unsigned long long natural;
#else
typedef unsigned __int64 natural;
#endif

/**
  Basical builtin text object
  */

#if defined(GC_DISABLED)
class gc_cleanup
{
};
#define GC_malloc_atomic malloc
#define GC_malloc malloc
#define GC_realloc realloc
#endif

class text : public gc_cleanup
{
 public:

 /**
   Dynamic size of the string
   */

 int len;
 char *data;

 text *op;

 text();

 /**
   Constructor from a C string
   */

 text(const char *);

 /**
   Constructor for the = operator
   */
 text(const text &);

 text(text *);

 text(char);

 /**
   Constructor: convert an integer into a string of digits
   */
 text(int);
 text(unsigned int);
 text(natural);

 /**
   Constructor: convert a number into a string of digits
   */

 text(double);
 text(bool);
 text(void *);
 ~text();

 void set(text *);
 void set(const text &);
 void set(const char *);
 void set(int);
 void set(unsigned int);
 void set(natural);
 void set(double);
 void set(bool);
 void set(char);


 /**
   Returns the current size of the string
   */
 inline int length() { return len; }


 /**
   Returns true if the text is a null string
   */
 inline int empty()  { return len == 0; }


 /**
   Converts to upper case
   */
 text upper();


 /**
   Converts to lower case
   */
 text lower();


 /**
   Converts the first letter to upper case (other ones unchanged)
   */
 text capitalize();


 /**
   Concatenates the string with the one in parameter.
   */
 void cat(const char *);
// void cat(const text &);

 bool in(const char *);
 bool in(const text &);


 /**
   Searches for the given string inside the text.
   An optional second parameter is the starting position in the text.
   Don't ignore case.
   Returns the position of the substring found inside the text.
   */
 int find(const char *, int = 0);


 /**
   Searches for the given string inside the text.
   An optional second parameter is the starting position in the text.
   Ignores case.
   */
 int findLex(const char *, int = 0);


 /**
   Static method.
   Compares two strings. Optionally limits the length of the strings,
   according to the third argument.
   Ignores case.
   Returns:
   -1   if the first is lesser than the second one.
   0    if the strings are identical.
   1    if the first is greater than the second one.
   */
 static int compareLex(const char *, const char *, int);

 /**
   Compares the text with a given strings.
   Ignores case.
   Returns:
   -1   if the text is lesser than the string.
   0    if the strings are identical.
   1    if the text is greater than the given string.
   */
 int compare(const char *);

 /**
   Compares the text with the given strings.
   Don't ignores case.
   Returns:
   -1   if the text is lesser than the string.
   0    if the strings are identical.
   1    if the text is greater than the given string.
   */
 int identical(const char *);

 /**
   Replaces inside the text, the first string by the second one.
   Don't ignores case.
   */
 void replace(const char *, const char *);

 /**
   Inserts a string inside the text at the given location (first argument).
   */
 void insert(int, const char *);

 text slice(int, int, int);
 const char * substr(int, int);      // as slice, return char pointer
 const char * substr(int);

 void splice(int, int, const char*, int);
 void splice(int a, int b, const text &t, int i);

 /**
   Removes trailing blank spaces and control characters.
   */
 text rTrim();

 /**
   Removes heading blank spaces and control characters.
   */
 text lTrim();

 /**
   Removes both heading and trailing blank spaces and control characters.
   */
 text trim();

 /**
   Builds a string from n occurences (the second parameter)
   of the string given as first parameter.
   */
 void fill(const char *, int);

 /**
   Builds a text from n occurences (the parameter) of the text,
   and returns it.
   */
 text dup(int);


 char toChar();

 /**
   Converts the text, if it is a string of digits, to an integer,
   and returns it.
   Returns 0 if no a number.
   */
 int toInt();

 /**
   As toInt().
   */
 int toInteger();

 /**
   Static function: converts a string into a number,
   return a double or 0 if not a string of digits.
   */
 static double toNumber(char *);

 /**
   Converts the text into a number,
   returns a number or 0 if the text is not a string of digits.
   */
 double toNumber();

 /**
   Converts the text, if it is a string of digits, to a natural,
   and returns it.
   Returns 0 if no a number.
   */
 unsigned long toNatural();

 /**
   Converts the text into a real number,
   returns a real or 0 if the text is not a string of digits.
   */
 inline double toReal()     { return toNumber(); }

 /**
   Returns the C string that the object holds.
   */
 inline char *toStr()       { return data; }

 /**
   Returns the C string that the object holds.
   */
 inline char *c_str() { return (char *) data; }

 /**
   Returns true if the text is a string of digits.
   */
 bool isNumber();

 /**
   Static function.
   Returns true if the given string is a string of digits.
   */
 static bool isNumber(const char *);

 bool operator==(const text &);
 bool operator==(const char *);
 bool operator==(int);
 bool operator==(double);
 bool operator==(bool);

 bool operator!=(const text &);
 bool operator!=(const char *);
 bool operator!=(int);
 bool operator!=(double);
 bool operator!=(bool);

 bool operator<=(const text &);
 bool operator<=(const char *);

 bool operator>=(const text &);
 bool operator>=(const char *);

 bool operator<(const text &);
 bool operator<(const char *);

 bool operator>(const text &);
 bool operator>(const char *);

 bool operator!();
 
 void operator=(const text &);
 void operator=(const char *);

 text operator+(const text &);
 text operator+(const char *);

 void operator+=(const text &);
 void operator+=(const char *);

 text &operator[](int);

 void setAt(int, const text &);
 text operator*(int);

};

std::ostream & operator<< (std::ostream &, text);
//std::ostream & operator<< (std::ostream &, text *);

inline bool operator==(const char *a, text b) { return b.operator==(a); }
inline bool operator==(bool a, text b) { return b.operator==(a); }
inline bool operator!=(const char *a, text b) { return b.operator!=(a); }
inline bool operator!=(bool a, text b) { return b.operator!=(a); }
inline bool operator<=(const char *a, text b) { return b.operator>(a); }
inline bool operator>=(const char *a, text b) { return b.operator<(a); }
inline bool operator<(const char *a, text b) { return b.operator>(a); }
inline bool operator>(const char *a, text b) { return b.operator<(a); }
inline text operator+(const char *a, text b) { return text(a) + b; }


#endif
